/*---------------------------------------------------------------------------\
| Project:          Generic LV2 PIC18F458 CAN node firmware
|
| File:             main.c
|
| Description:      Mainline routines.
|
| Global Routines:  Main()
|
| Local Routines:   None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/

//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the PicCore framework

#if CORE_VERSION != 5
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

#include "pwr.h"   // Includes for the imu firmware program

//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------

// flags to start adc sampling
bit sample_bus_current_flag;
bit sample_bus_voltage_flag;

// hold results from the ADC before sending in the CAN message
// I dont' realy need these I should get rid of them soon!!!!
uint8_t buss_i_temp_h;
uint8_t buss_i_temp_l;
uint8_t buss_i_max_h;
uint8_t buss_i_max_l;
uint8_t buss_i_avg_h;
uint8_t buss_i_avg_l;
uint8_t buss_i_min_h;
uint8_t buss_i_min_l;

uint8_t buss_v_temp_h;
uint8_t buss_v_temp_l;
uint8_t buss_v_max_h;
uint8_t buss_v_max_l;
uint8_t buss_v_avg_h;
uint8_t buss_v_avg_l;
uint8_t buss_v_min_h;
uint8_t buss_v_min_l;

// flags for when a can message is ready to be sent
bit umb_plug_status_message_flag;
bit umb_rocket_ready_message_flag;
bit umb_shore_power_message_flag;
bit umb_bus_current_message_flag;
bit umb_bus_voltage_message_flag;
bit umb_avionic_cmd_message_flag;

//test counter
uint16_t test_counter;
uint8_t adc_sensor_number;
uint32_t bus_voltage_counter;
uint32_t bus_current_counter;

// these vars hold the adc values to make min,max,avg calcualtions easy.
uint16_t bus_voltage_temp_16;
uint16_t bus_current_temp_16;

uint16_t bus_voltage_max_16;
uint16_t bus_current_max_16;

uint16_t bus_voltage_avg_16;
uint16_t bus_current_avg_16;

uint16_t bus_voltage_min_16;
uint16_t bus_current_min_16;

// defines for pic pins specific to this application
#DEFINE RC0 IPS_SWITCH

#DEFINE RC2 IPS_D2_ERROR // PIC INPUT
#DEFINE RC1 IPS_D1_ERROR // PIC INPUT
#DEFINE RC4 ROCKET_READY // PIC OUTPUT
#DEFINE RC3 UMB //THIS IS AN INPUT PIN ON THE PIC IF IT IS HIGH THE CORD IS PLUGGED IN
#DEFINE RDO VOL_PLUS_5 // THIS IS A PIC OUT PUT AND IS USED TO POWER THE INA168A CURRENT AMPLIFIER
#DEFINE AN2 UMB_VOLTAGE // UMB VOLTAGE AS SEEN BY THE ADC CHANNEL 2 ADD COUNTER TO  






/*---------------------------------------------------------------------------\
| Function:     main
|
| Purpose:      Run all Mainline code
|
| Discussion:   Doesn't do much, huh?
|
| Expects:      Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
    static CanMessage_t tempMsg;
    // static CanMessage_t  tempMsg2;

    // Initialize everything and its brother

    Initialize();


    // Loop for ever...and......ev....er....and.......ev............er.

    while (TRUE)
    {

        if(umb_bus_current_message_flag)
        {
          tempMsg.header.all = BUS_CURRENT_DATA;   // set can node header
          if( bus_current_counter == 1)
          {
            // implies first reading hence max,min,avg make no sence
            // note buss_i_max is an 8bit var, 8bit = 16bit I assume the 8bit ends
            // up with the lower byte.
            bus_current_max_16 = bus_current_temp_16;
            bus_current_avg_16 = bus_current_temp_16;
            bus_current_min_16 = bus_current_temp_16;
            tempMsg->data[0] = bus_current_temp_16 >> 8; //max
            tempMsg->data[1] = bus_current_temp_16;
            tempMsg->data[2] = bus_current_temp_16 >> 8; // avg
            tempMsg->data[3] = bus_current_temp_16;
            tempMsg->data[4] = bus_current_temp_16 >> 8; // min
            tempMsg->data[5] = bus_current_temp_16;
          } else {
            // replace high and low values as nessessary
            bus_current_avg_16 = (bus_current_avg_16 + bus_current_temp_16) / bus_current_counter;
            if(bus_current_max_16 < bus_current_temp_16){
              bus_current_max_16 = bus_current_temp_16;
            } else if(bus_current_min_16 > bus_current_temp_16){
              bus_current_min_16 = bus_current_temp_16;
            }
            tempMsg->data[0] = bus_current_max_16 >> 8; //max
            tempMsg->data[1] = bus_current_max_16;
            tempMsg->data[2] = bus_current_avg_16 >> 8; // avg
            tempMsg->data[3] = bus_current_avg_16;
            tempMsg->data[4] = bus_current_min_16 >> 8; // min
            tempMsg->data[5] = bus_current_min_16;

          }
          ++bus_current_counter; // what is this counter for? to calc avg (number of samples taken)
          umb_bus_current_message_flag = 0;
          SendCanMessage(&tempMsg);
        } // end bus current message flag stuff



        if(umb_bus_voltage_message_flag)
        {
            if(bus_voltage_counter == 1)
            {
              bus_voltage_max_16 = bus_voltage_temp_16;
              bus_voltage_avg_16 = bus_voltage_temp_16;
              bus_voltage_min_16 = bus_voltage_temp_16;
              tempMsg->data[0] = bus_voltage_temp_16 >> 8; //max
              tempMsg->data[1] = bus_voltage_temp_16;
              tempMsg->data[2] = bus_voltage_temp_16 >> 8; // avg
              tempMsg->data[3] = bus_voltage_temp_16;
              tempMsg->data[4] = bus_voltage_temp_16 >> 8; // min
              tempMsg->data[5] = bus_voltage_temp_16;
              // implies first reading hence max,min,avg make no sence
            } else {
        // replace high and low values as nessessary
              bus_voltage_avg_16 = (bus_voltage_avg_16 + bus_voltage_temp_16)/bus_voltage_counter;
              if(bus_voltage_max_16 < bus_voltage_temp_16)
              {
                bus_voltage_max_16 = bus_voltage_temp_16;
              } else if (bus_voltage_min_16 > bus_voltage_temp_16){
                bus_voltage_min_16 = bus_voltage_temp_16;
              }
          // calc and save the averages
            tempMsg.header.all = BUS_VOLTAGE_DATA;
            tempMsg->data[0] = bus_voltage_max_16 >> 8; //max
            tempMsg->data[1] = bus_voltage_max_16;
            tempMsg->data[2] = bus_voltage_avg_16 >> 8; // avg
            tempMsg->data[3] = bus_voltage_avg_16;
            tempMsg->data[4] = bus_voltage_min_16 >> 8; // min
            tempMsg->data[5] = bus_voltage_min_16;
          }
            ++bus_voltage_counter;
            SendCanMessage(&tempMsg);
            umb_bus_voltage_message_flag = 0;
        }

        if(canMessageAvailable)
        {
            GetCanMessage(&tempMsg);
            Parse_Can_Message(&tempMsg);
        }
    ++test_counter;
   }

}

/*---------------------------------------------------------------------------\
| Function:     Initialize
|
| Purpose:      Initialize everything. Really.
|
| Discussion:   Did I mention initialization?
|
| Expects:      To be called after a RESET - eg, registers reset to known
|               values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

    // Setting up the CAN bus peripheral: We're passing constants
    // to the initialization routine so that we can just set the
    // initialize initialization :) here.

    const CanHeader_t canMaskB0 = (0x07FF << 5);
    const CanHeader_t canMaskB1 = (0x07FF << 5);
    const CanHeader_t CanFilterB0F0 = (0x07FC <<5);
    const CanHeader_t CanFilterB0F1 = (0x0201 << 5);
    const CanHeader_t CanFilterB1F2 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F3 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F4 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F5 = (0x0000 << 5);

    // Get the outputs stabilized FIRST.
    // PORTA RA0 = IMU TEMP
    // PORTB RB3 = CAN RX RB2 = CAN TX RB0 = INT0
    // PORTC RC0 = /RD RC1 = /WR RC2 = TEST
    // PROTD = DATA BUS
    // PORTE RE0 = /CS RE2 = HBEN

    InitializePorts (  0b00000000,           // port_a_latch
                       0b00000001,           // port_a_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RA0/AN0/CVref     = out = IMU Temperature
                      // ||||||+------------ RA1/AN1           = out =
                      // |||||+------------- RA2/AN2/Vref-     = out =
                      // ||||+-------------- RA3/AN3/Vref+     = in  =
                      // |||+--------------- RA4/T0CKI         = out =
                      // ||+---------------- RA5/AN4/SS*/LVDIN = out =
                      // |+----------------- RA6/OSC2/CLKOUT   = out =
                      // +------------------ <unimplemented>
                      // WARNING: DON'T FORGET TO INTIALIZE THE ADC CORRECTLY
                      // TO ENABLE PORT A TO BE OUTPUTS.

                       0b00000000,           // port_b_latch
                       0b00001001,           // port_b_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RB0/INT0          = out = /INT from MAX197
                      // ||||||+------------ RB1/INT1          = out =
                      // |||||+------------- RB2/CANTX         = out = CANTX
                      // ||||+-------------- RB3/CANRX         = in  = CANRX
                      // |||+--------------- RB4               = out =
                      // ||+---------------- RB5/PGM           = out =
                      // |+----------------- RB6/PGC           = out = ICD2 PGC
                      // +------------------ RB7/PGD           = out = ICD2 PGD

                       0b00000011, // port_c_latch
                       0b00000000, // port_c_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RC0/T1OSO/T1CKI   = out = /RD from MAX197
                      // ||||||+------------ RC1/T1OSI         = out = /WR to MAX 197
                      // |||||+------------- RC2/CCP1X         = out = test accelerometers
                      // ||||+-------------- RC3/SCK/SCL       = in  =
                      // |||+--------------- RC4/SDI/SDA       = out =
                      // ||+---------------- RC5/SDO           = out =
                      // |+----------------- RC6/TX/CK         = out =
                      // +------------------ RC7/RX/DT         = out =

///ANDREW CHECK THIS PLEASE AND ERASE THIS WHEN DONE
                       0b00000000, // port_d_latch
                       0b00101111, // port_d_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RD0/PSP0/C1IN+    = in =  Board has power
                      // ||||||+------------ RD1/PSP1/C1IN-    = in =  UMB is plugged in
                      // |||||+------------- RD2/PSP2/C2IN+    = in =  pwr2 power supply 2 on/off status
                      // ||||+-------------- RD3/PSP3/C2IN-    = in =  pwr1 power supply 1 on/off status
                      // |||+--------------- RD4/PSP4/ECCP1/P1A= out = Power Enabled
                      // ||+---------------- RD5/PSP5/P1B      = in =  UMB Voltage check
                      // |+----------------- RD6/PSP6/P1C      = out = Rocket Ready off=0 on=1
                      // +------------------ RD7/PSP7/P1D      = X =

                       0b00000001, // port_e_latch
                       0b00000000, // port_e_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RE0/RD*/AN5       = out = /CS from MAX197
                      // ||||||+------------ RE1/WR*/AN6/C1OUT = out =
                      // |||||+------------- RE2/CS*/AN7/C2OUT = out = HBEN from MAX197
                      // ||||+-------------- <unimplimented>
                      // |||+--------------- <unimplimented>
                      // ||+---------------- <unimplimented>
                      // |+----------------- <unimplimented>
                      // +------------------ <unimplimented>

                       ( PORTS_PORTB_PULLUPS_ON
                         | PORTS_INT0_FALLING_EDGE
                         | PORTS_INT1_RISING_EDGE
                         | PORTS_INT0_INTS_ENABLED
                         | PORTS_INT1_INTS_DISABLED ) );


    // Initialize the ADC peripheral. Note that it directly affects PORT A
    // and PORT E pins due to the ADC taking over the TRIS register.

    initialize_adc ( ADC_ENABLED
                     | ADC_JUSTIFY_RIGHT
                     | ADC_CHANNEL_0
                     | ADC_PORT_1_CH
                     | ADC_INTS_ENABLED );
    // initalize adc channel to channel 0
         ADC_CHS_2 = 0;
         ADC_CHS_1 = 0;
         ADC_CHS_0 = 0;
   // intialize can message data
         buss_i_max_h = 0;
         buss_i_max_l = 0;
         buss_i_avg_h = 0;
         buss_i_avg_l = 0;
         buss_i_min_h = 0;
         buss_i_min_l = 0;

         buss_v_max_h = 0;
         buss_v_max_l = 0;
         buss_v_avg_h = 0;
         buss_v_avg_l = 0;
         buss_v_min_h = 0;
         buss_v_min_l = 0;
     // intialize the sample counters
         bus_voltage_counter = 1;
         bus_current_counter = 1;

    // Turn off the serial port

    InitializeSerial (  SERIAL_TRANSMIT_DISABLE
                        | SERIAL_TRANSMIT_DISABLE_INTS
                        | SERIAL_RECEIVE_DISABLE
                        | SERIAL_RECEIVE_DISABLE_INTS
                        | SERIAL_BITRATE_9600 );

    // Turn on timer2, HP interrupts @ 1KHz

    initialize_timer2 (TIMER2_5KHZ | TIMER2_ENABLE_INTS);

    // Turn on CAN: See const variable declarations above
    // for the values. Note that as soon as we initialize the
    // the bus, we can start to receive messages, so be sure
    // that interrupts are enabled VERY SOON after this call
    // (No more than about a  message length, which at top
    // speed is about 1/22KHz = 45us which at 20MHz is only
    // 250 instructions.

    InitializeCan ( CAN_BITRATE_1MBPS,
                    canMaskB0,          // Mask RXB0
                    canMaskB1,          // Mask RXB1
                    CanFilterB0F0,      // Filter RXB0 F0
                    CanFilterB0F1,      // Filter RXB0 F1
                    CanFilterB1F2,      // Filter RXB1 F2
                    CanFilterB1F3,      // Filter RXB1 F3
                    CanFilterB1F4,      // Filter RXB1 F4
                    CanFilterB1F5);     // Filter RXB1 F5

    // ---------------------------------------------------------------------
    // PICCORE: ALL INITIALIZATION THAT MUST BE DONE PRIOR TO INTERRUPTS
    // BEING ENABLED MUST BE DONE PRIOR TO "InitializeInterrupts()".

    InitializeInterrupts();

    // PICCORE: INTERRUPTS ARE INITIALIZED; FEEL FREE TO INITIALIZE THINGS
    // WHICH REQUIRE INTERRUPTS TO BE RUNNING AFTER THIS POINT.
    // ---------------------------------------------------------------------
}



/*---------------------------------------------------------------------------\
| Function:     Parse_Can_Message
|
| Purpose:      Interpet can message and act on it.
|
| Discussion:   see purpose
|
| Expects:      a can message with directions in it
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Parse_Can_Message(CanMessage_t* receivedMsg)
{
    switch(receivedMsg->header.all)
    {
    // begin imu instruction / manipulation
      case AVIONIC_COMMANDS:
        if(receivedMsg->data[0] == 0xAA && receivedMsg->data[1] == 0xD5 && receivedMsg->data[2] == 0x23 ){
           PORTD & 11101111; // is this right
        }
        break;
      case ROCKET_READY_ON:
        ROCKET_READY_BIT = 1;
        break;

      case ROCKET_READY_OFF:
        ROCKET_READY_BIT = 0;
        break;

      default:
        break;
    } // end ID switch


} // end Parse_Can_Message

/*---------------------------------------------------------------------------\
| Function:     ADCApplicationHandler
|
| Purpose:      Set the temp_can_msg_flag flag when pic adc is done its conversion
|
| Discussion:   See purpose
|
| Expects:      The ADON=1, GODONE previously asserted.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/12/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

#if INTS_ADC_APPLICATION_HOOK == INTS_HOOK_ENABLED

void adc_application_handler (void)
{
  switch(adc_sensor_number)
  {
    case 0: // current
      HBEN = 0;
      bus_current_temp_16 = ADRESL;
      HBEN = 1;
      bus_current_temp_16 |= ( (uint16_t)ADRESH << 8);
      adc_sensor_number = 1;
      umb_bus_current_message_flag = 1;
      break;

    case 1: // voltage
      HBEN = 0;
      bus_voltage_temp_16 = ADRESL;
      HBEN = 1;
      bus_voltage_temp_16 |= ( (uint16_t)ADRESH << 8);
      adc_sensor_number =
 0;
      umb_bus_voltage_message_flag = 1;
      break;
    default:
      // throw Warring
      break;
  } // end of take sample switch

   switch(adc_sensor_number){
     case 0:
       ADC_CHS_2 = 0;
       ADC_CHS_1 = 0;
       ADC_CHS_0 = 0;
       break;
     case 1:
       ADC_CHS_2 = 0;
       ADC_CHS_1 = 0;
       ADC_CHS_0 = 1;
       break;
     defalut:
       // throw_warrning
       break;
  } // end switch
}

#endif


/*---------------------------------------------------------------------------\
| Function:     Application_Timer2_Handler
|
| Purpose:      Check and see what if any adc samples need to be taken
|
| Discussion:
|
|
|
| Expects:      The four global sample rate variables to be set.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

#if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED

void timer2_application_handler ()
{
  GODONE = 1;
} // end Application_Timer2_Handler

#endif
