/*---------------------------------------------------------------------------\
| Project:  imu LV2 PIC18F458 CAN node firmware
|
| File:     imu.h
|
| Descr.:   Demo.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _PWR_H     // Don't include more than once (last line = #endif)
#define _PWR_H

//------------------------------------------------------------------------
// Typedefs
//------------------------------------------------------------------------


enum PWR_WARNING_MESSAGES
{
    ADC_CHANNEL_OUT_OF_RANGE,
    Some_other_message_to_keep_syntax_correct
};


//------------------------------------------------------------------------
// #defines
//------------------------------------------------------------------------
// #define the pins of the 18F458
#define BOARD_POWER_STATUS RD0 //PIN 38
#define UMB_PLUG_STATUS RD1          //PIN 39
#define POWER_SUPP_1_STATUS RD2      //PIN 40
#define POWER_SUPP_2_STATUS RD3      //PIN 41
#define ENABLE_POWER_SUPPIES RD4     //PIN 2
#define UMB_VOLTAGE_STATUS RD5       //PIN 3
#define ROCKET_READY_BIT RD6         //PIN 4
#define D7 RD7       //PIN 5 DONT CARE
#define CANRX RB3    //PIN 11
#define CANTX RB2    //PIN 10
#define HBEN RE2     //PIN 27

#define ADC_CHS_2 CHS2 // MSB for adc channel select
#define ADC_CHS_1 CHS1 // middle bit for adc channel select
#define ADC_CHS_0 CHS0 // LSB for adc channel select

#define BUS_VOLTAGE_IN AN1  //PIN 4
#define BUS_CURRENT_IN AN0  //PIN 3

// defines for types of can messages
#define UMB_STATS_UNPLUGED 0b0010000001000000
#define UMB_STATS_PLUGED 0b0010000001100000
#define ROCKET_READY_OFF 0b0010000010000000
#define ROCKET_READY_ON 0b0010000010100000
#define SHORE_POWER_OFF 0b0010000011000000
#define SHORE_POWER_ON 0b0010000011100000

#define BUS_CURRENT_DATA 0b0010010100000110
#define BUS_VOLTAGE_DATA 0b0010010100100110
#define AVIONIC_COMMANDS 0b0010011000000011

//defines for portD masks
#define BOARD_POWER_MASK 0b00000001
#define UMB_PLUG_MASK 0b00000010
#define PWR_2_MASK 0b00000100
#define PWR_1_MASK 0b00001000
#define SHORE_POWER_MASK 0b00100000


//------------------------------------------------------------------------
// Function declarations
//------------------------------------------------------------------------
void Initialize (void);

void Parse_Can_Message(CanMessage_t* receivedMsg);

void adc_channel_select(void);


//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------
//These globals are used to define the sample rates for IMU and Temp and Pressure
// hold results from the ADC before sending in the CAN message
extern uint8_t buss_i_temp_h;
extern uint8_t buss_i_temp_l;
extern uint8_t buss_i_max_h;
extern uint8_t buss_i_max_l;
extern uint8_t buss_i_avg_h;
extern uint8_t buss_i_avg_l;
extern uint8_t buss_i_min_h;
extern uint8_t buss_i_min_l;

extern uint8_t buss_v_temp_h;
extern uint8_t buss_v_temp_l;
extern uint8_t buss_v_max_h;
extern uint8_t buss_v_max_l;
extern uint8_t buss_v_avg_h;
extern uint8_t buss_v_avg_l;
extern uint8_t buss_v_min_h;
extern uint8_t buss_v_min_l;

// flags for when a can message is ready to be sent
extern bit umb_status_message_flag;
extern bit umb_rocket_ready_message_flag;
extern bit umb_shore_power_message_flag;
extern bit umb_bus_current_message_flag;
extern bit umb_bus_voltage_message_flag;

// flags to start adc
extern bit sample_bus_current_flag;
extern bit sample_bus_voltage_flag;
extern uint16_t bus_voltage_temp_16;
extern uint16_t bus_current_temp_16;

extern uint16_t bus_voltage_max_16;
extern uint16_t bus_current_max_16;

extern uint16_t bus_voltage_avg_16;
extern uint16_t bus_current_avg_16;

extern uint16_t bus_voltage_min_16;
extern uint16_t bus_current_min_16;

//test counter
extern uint16_t test_counter;
extern uint8_t adc_sensor_number;
extern uint32_t bus_current_counter;
extern uint32_t bus_voltage_counter;
//-----------------------------------------------------------------------
// Module-wide variable declarations
//------------------------------------------------------------------------

// None


#endif              // _PWR_H


